@@ -74,8 +74,8 @@ load_interface (const gchar *top_widget)
     xml = glade_xml_new (GLADE_FILE, top_widget, GETTEXT_PACKAGE);
 
     if (!xml) {
-	g_warning (_("Couldn't load glade file"));
-	return NULL;
+        g_warning (_("Couldn't load glade file"));
+        return NULL;
     }
     
     return xml;
@@ -97,7 +97,7 @@ groups_filter_fn (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 
 static void 
 setup_groups_treeview (GbfProjectModel *model,
-		       GtkWidget       *view,
+                       GtkWidget       *view,
                        const gchar     *select_group)
 {
     GtkTreeModel *filter;
@@ -109,11 +109,10 @@ setup_groups_treeview (GbfProjectModel *model,
 
     filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
     gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
-					    groups_filter_fn, NULL, NULL);
+                                            groups_filter_fn, NULL, NULL);
     gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
     g_object_unref (filter);
-    gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
-
+    
     /* select default group */
     if (select_group && gbf_project_model_find_id (model, &iter,
                                                    GBF_TREE_NODE_GROUP, select_group)) {
@@ -122,10 +121,11 @@ setup_groups_treeview (GbfProjectModel *model,
         gtk_tree_model_filter_convert_child_iter_to_iter (
             GTK_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
         path = gtk_tree_model_get_path (filter, &iter_filter);
-
+        gtk_tree_view_expand_to_path (GTK_TREE_VIEW (view), path);
     } else {
         GtkTreePath *root_path;
 
+        gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
         root_path = gbf_project_model_get_project_root (model);
         if (root_path) {
             path = gtk_tree_model_filter_convert_child_path_to_path (
@@ -137,7 +137,8 @@ setup_groups_treeview (GbfProjectModel *model,
     }
     
     gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL, TRUE, 0.5, 0.0);
+    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL,
+                                  TRUE, 0.5, 0.0);
     gtk_tree_path_free (path);
 }
 
@@ -182,10 +183,11 @@ error_dialog (GtkWindow *parent, const gchar *msg, ...)
     gtk_widget_destroy (dialog);
 }
 
-void 
+gchar*
 gbf_project_util_new_group (GbfProjectModel *model,
-			    GtkWindow       *parent,
-			    const gchar     *default_group)
+                            GtkWindow       *parent,
+                            const gchar     *default_group,
+                            const gchar     *default_group_name_to_add)
 {
     GladeXML *gui;
     GtkWidget *dialog, *group_name_entry, *ok_button;
@@ -193,15 +195,16 @@ gbf_project_util_new_group (GbfProjectModel *model,
     gint response;
     GbfProject *project;
     gboolean finished = FALSE;
-    
-    g_return_if_fail (model != NULL);
+    gchar *new_group = NULL;
+
+    g_return_val_if_fail (model != NULL, NULL);
     
     project = gbf_project_model_get_project (model);
     if (!project)
-        return;
+        return NULL;
 
     gui = load_interface ("new_group_dialog");
-    g_return_if_fail (gui != NULL);
+    g_return_val_if_fail (gui != NULL, NULL);
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "new_group_dialog");
@@ -210,15 +213,21 @@ gbf_project_util_new_group (GbfProjectModel *model,
     ok_button = glade_xml_get_widget (gui, "ok_button");
     
     /* set up dialog */
+    if (default_group_name_to_add)
+        gtk_entry_set_text (GTK_ENTRY (group_name_entry),
+                            default_group_name_to_add);
     g_signal_connect (group_name_entry, "changed",
                       (GCallback) entry_changed_cb, ok_button);
-    gtk_widget_set_sensitive (ok_button, FALSE);
+    if (default_group_name_to_add)
+        gtk_widget_set_sensitive (ok_button, TRUE);
+    else
+        gtk_widget_set_sensitive (ok_button, FALSE);
     
     setup_groups_treeview (model, groups_view, default_group);
     gtk_widget_show (groups_view);
     
     if (parent) {
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
     }
     
     /* execute dialog */
@@ -250,8 +259,6 @@ gbf_project_util_new_group (GbfProjectModel *model,
                                       err->message);
                         g_error_free (err);
                     }
-            
-                    g_free (new_group);
                     g_free (parent_id);
                     finished = TRUE;
                     
@@ -273,6 +280,7 @@ gbf_project_util_new_group (GbfProjectModel *model,
     /* destroy stuff */
     gtk_widget_destroy (dialog);
     g_object_unref (gui);
+    return new_group;
 }
 
 enum {
@@ -325,10 +333,11 @@ build_types_store (GbfProject *project)
     return store;
 }
 
-void 
+gchar* 
 gbf_project_util_new_target (GbfProjectModel *model,
-			     GtkWindow       *parent,
-			     const gchar     *default_group)
+                             GtkWindow       *parent,
+                             const gchar     *default_group,
+                             const gchar     *default_target_name_to_add)
 {
     GladeXML *gui;
     GtkWidget *dialog, *target_name_entry, *ok_button;
@@ -338,15 +347,16 @@ gbf_project_util_new_target (GbfProjectModel *model,
     gint response;
     GbfProject *project;
     gboolean finished = FALSE;
+    gchar *new_target = NULL;
     
-    g_return_if_fail (model != NULL);
+    g_return_val_if_fail (model != NULL, NULL);
     
     project = gbf_project_model_get_project (model);
     if (!project)
-        return;
+        return NULL;
 
     gui = load_interface ("new_target_dialog");
-    g_return_if_fail (gui != NULL);
+    g_return_val_if_fail (gui != NULL, NULL);
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "new_target_dialog");
@@ -356,9 +366,15 @@ gbf_project_util_new_target (GbfProjectModel *model,
     ok_button = glade_xml_get_widget (gui, "ok_button");
     
     /* set up dialog */
+    if (default_target_name_to_add)
+        gtk_entry_set_text (GTK_ENTRY (target_name_entry),
+                            default_target_name_to_add);
     g_signal_connect (target_name_entry, "changed",
                       (GCallback) entry_changed_cb, ok_button);
-    gtk_widget_set_sensitive (ok_button, FALSE);
+    if (default_target_name_to_add)
+        gtk_widget_set_sensitive (ok_button, TRUE);
+    else
+        gtk_widget_set_sensitive (ok_button, FALSE);
     
     setup_groups_treeview (model, groups_view, default_group);
     gtk_widget_show (groups_view);
@@ -390,7 +406,7 @@ gbf_project_util_new_target (GbfProjectModel *model,
     gtk_combo_box_set_active (GTK_COMBO_BOX (target_type_combo), 0);
 
     if (parent) {
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
     }
     
     /* execute dialog */
@@ -418,8 +434,6 @@ gbf_project_util_new_target (GbfProjectModel *model,
                 }
                 
                 if (data && type) {
-                    gchar *new_target = NULL;
-                    
                     gtk_widget_hide (dialog);
                     
                     group_id = g_strdup (data->id);
@@ -431,8 +445,6 @@ gbf_project_util_new_target (GbfProjectModel *model,
                                       err->message);
                         g_error_free (err);
                     }
-            
-                    g_free (new_target);
                     g_free (group_id);
                     g_free (type);
                     finished = TRUE;
@@ -456,6 +468,7 @@ gbf_project_util_new_target (GbfProjectModel *model,
     g_object_unref (types_store);
     gtk_widget_destroy (dialog);
     g_object_unref (gui);
+    return new_target;
 }
 
 static gboolean
@@ -477,34 +490,47 @@ targets_filter_fn (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 static void 
 setup_targets_treeview (GbfProjectModel *model,
                         GtkWidget       *view,
-                        const gchar     *select_target)
+                        const gchar     *select_target,
+                        const gchar     *select_group)
 {
     GtkTreeModel *filter;
     GtkTreeIter iter, iter_filter;
+    GtkTreePath *path = NULL;
     
     g_return_if_fail (model != NULL);
     g_return_if_fail (view != NULL && GBF_IS_PROJECT_VIEW (view));
 
     filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
     gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
-					    targets_filter_fn, NULL, NULL);
+                                            targets_filter_fn, NULL, NULL);
     gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
     g_object_unref (filter);
-    gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
 
     /* select default target */
     if (select_target) {
         if (gbf_project_model_find_id (model, &iter,
                                        GBF_TREE_NODE_TARGET, select_target)) {
-            GtkTreePath *path;
-            
             gtk_tree_model_filter_convert_child_iter_to_iter (
                 GTK_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
             path = gtk_tree_model_get_path (filter, &iter_filter);
-            gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
-            gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL, TRUE, 0.5, 0.0);
-            gtk_tree_path_free (path);
         }
+    } else if (select_group) {
+        if (gbf_project_model_find_id (model, &iter,
+                                       GBF_TREE_NODE_GROUP, select_group)) {
+            gtk_tree_model_filter_convert_child_iter_to_iter (
+                  GTK_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
+            path = gtk_tree_model_get_path (filter, &iter_filter);
+        }
+    }
+    if (path)
+    {
+        gtk_tree_view_expand_to_path (GTK_TREE_VIEW (view), path);
+        gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+        gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL,
+                                      TRUE, 0.5, 0.0);
+        gtk_tree_path_free (path);
+    } else {
+        gtk_tree_view_expand_all (GTK_TREE_VIEW (view));
     }
 }
 
@@ -537,11 +563,12 @@ browse_button_clicked_cb (GtkWidget *widget, gpointer user_data)
     }
 }
 
-void 
+gchar* 
 gbf_project_util_add_source (GbfProjectModel *model,
-			     GtkWindow       *parent,
-			     const gchar     *default_target,
-			     const gchar     *default_uri)
+                             GtkWindow       *parent,
+                             const gchar     *default_target,
+                             const gchar     *default_group,
+                             const gchar     *default_uri)
 {
     GladeXML *gui;
     GtkWidget *dialog, *source_file_entry;
@@ -551,15 +578,16 @@ gbf_project_util_add_source (GbfProjectModel *model,
     GbfProject *project;
     gboolean finished = FALSE;
     gchar *project_root;
+    gchar *new_source = NULL;
     
-    g_return_if_fail (model != NULL);
+    g_return_val_if_fail (model != NULL, NULL);
     
     project = gbf_project_model_get_project (model);
     if (!project)
-        return;
+        return NULL;
 
     gui = load_interface ("add_source_dialog");
-    g_return_if_fail (gui != NULL);
+    g_return_val_if_fail (gui != NULL, NULL);
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "add_source_dialog");
@@ -583,11 +611,11 @@ gbf_project_util_add_source (GbfProjectModel *model,
     g_object_set_data_full (G_OBJECT (browse_button), "root",
                             project_root, g_free);
     
-    setup_targets_treeview (model, targets_view, default_target);
+    setup_targets_treeview (model, targets_view, default_target, default_group);
     gtk_widget_show (targets_view);
     
     if (parent) {
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
     }
 
     if (default_target)
@@ -609,8 +637,6 @@ gbf_project_util_add_source (GbfProjectModel *model,
                 data = gbf_project_view_find_selected (GBF_PROJECT_VIEW (targets_view),
                                                        GBF_TREE_NODE_TARGET);
                 if (data) {
-                    gchar *new_source = NULL;
-                    
                     gtk_widget_hide (dialog);
                     
                     target_id = g_strdup (data->id);
@@ -625,8 +651,6 @@ gbf_project_util_add_source (GbfProjectModel *model,
                                       err->message);
                         g_error_free (err);
                     }
-
-                    g_free (new_source);
                     g_free (source_file);
                     g_free (target_id);
                     finished = TRUE;
@@ -635,7 +659,7 @@ gbf_project_util_add_source (GbfProjectModel *model,
                     error_dialog (parent, _("No target has been selected"));
                 
                 }
-	
+                
                 break;
             }
             default:
@@ -647,5 +671,5 @@ gbf_project_util_add_source (GbfProjectModel *model,
     /* destroy stuff */
     gtk_widget_destroy (dialog);
     g_object_unref (gui);
+    return new_source;
 }
-
